"""
This module holds all of the functions and classes necessary for 
initiating a JSONRPC-TCP Server.
"""
from __future__ import print_function

import logging
import threading
import socket
import time
import sys
import types
import traceback
from jsonrpctcp.handler import Handler
from jsonrpctcp import config
from jsonrpctcp import logger
from jsonrpctcp import history
from jsonrpctcp.errors import ProtocolError
from jsonrpctcp.errors import JSONRPC_ERRORS, EncryptionMissing
from inspect import isclass

try:
    import json
except ImportError:
    import simplejson as json

class Server(object):
    """
    This class is the basic Server object. It should be instantiated
    with a (host, port) tuple (and an optional handler), and then
    the Handler subclasses / functions should be attached through the
    add_handler method.
    """

    _shutdown = False

    def __init__(self, addr, handler=None, pool=10):
        if config.secret and not config.crypt:
            raise EncryptionMissing('No encrpytion library found.')
        self.addr = addr
        self.socket = None
        self.threads = []
        # Pool not actually implemented yet
        self.pool = pool
        self.json_request = JSONRequest(self)
        if handler:
            assert hasattr(handler, '__call__') or \
                issubclass(handler, Handler)
            self.json_request.add_handler(handler)
        
    def serve(self):
        """
        This starts the server -- it blocks, so if there are other
        tasks that need to be performed after the server is started,
        threading / multiprocessing will need to be employed.
        """
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self.socket.bind(self.addr)
        self.socket.listen(config.max_queue)
        self.wait()
        
    def wait(self):
        """ The principle wait cycle. """
        while True:
            if self._shutdown:
                break            
            clientsock, addr = self.socket.accept()
            args = (clientsock, addr)
            target = self.json_request.process
            thread = threading.Thread(target=target, args=args)
            thread.daemon = True
            thread.start()
            self.threads.append(thread)
            self.check_threads()
            
        sys.stdout.write('Shutting down...')
        for thread in self.threads:
            thread.join()
        sys.stdout.write('done.\n')

    def shutdown(self):
        """
        Attempts to shutdown the server.
        TODO: Make this work quickly and properly.
        """
        self._shutdown = True
        self.socket.close()
        
    def check_threads(self):
        """
        Check the thread list for dead threads and finished
        threads.
        """
        for thread in self.threads:
            if not thread.is_alive():
                thread.join()
                self.threads.remove(thread)
    
    def add_handler(self, method, name=None):
        """ Just a wrapper around JSONRequest.add_handler """
        self.json_request.add_handler(method, name)
            
class JSONRequest(object):
    """
    This is the class that handles individual requests passed
    from the server wait cycle.
    """

    def __init__(self, server):
        self.server = server
        self.handlers = {}

    def add_handler(self, method, name=None):
        """
        Attach a handler to the request object. It must be either
        callable, or a subclass of Handler.
        """
        if isinstance(method, Handler):
            # assert issubclass(method, Handler)
            # If it's an actual Handler subclass
            handler_instance = method
            for hname, method in handler_instance._handlers.items():
                if name:
                    hname = '%s.%s' % (name, hname)
                self.handlers[hname] = method
        else:
            if not name:
                name = method.__name__
            assert hasattr(method, '__call__')
            self.handlers[name] = method
            
    def get_handler(self, name):
        """ Check for an attached handler and return it. """
        if name in self.handlers:
            return self.handlers[name]
        return None
                
    def process(self, sock, addr):
        """ Just a wrapper for ProcessRequest. """
        request = ProcessRequest(self)
        request.process(sock, addr)
        
class ProcessRequest(object):
    """
    This is the class that handles an actual request, passing it through
    the Handler and parsing the response.
    """

    socket_error = False

    def __init__(self, json_request):
        self.json_request = json_request
        self.socket = None
        self.client_address = None
        
    def process(self, sock, addr):
        """
        Retrieves the data stream from the socket and validates it.
        """
        self.socket = sock
        self.socket.settimeout(config.timeout)
        self.client_address = addr
        requestlines = []
        while True:
            data = self.get_data()
            if not data: 
                break
            data = data.decode()
            requestlines.append(data)
            if len(data) < config.buffer: 
                break
        request = ''.join(requestlines)
        response = ''
        crypt_error = False
        if config.secret:
            crypt = config.crypt.new(config.secret)
            try:
                request = crypt.decrypt(request)
            except ValueError:
                crypt_error = True
                error = ProtocolError(-32700, 'Could not decrypt request.')
                response = json.dumps(error.generate_error())
        history.request = request
        logger.debug('SERVER | REQUEST: %s' % request)
        if self.socket_error:
            self.socket.close()
        else:
            if not crypt_error:
                response = self.parse_request(request)
            history.response = response
            logger.debug('SERVER | RESPONSE: %s' % response)
            if config.secret:
                length = config.crypt_chunk_size
                pad_length = length - (len(response) % length)
                response = crypt.encrypt('%s%s' % (response, ' '*pad_length))
            self.socket.send(response.encode())
        self.socket.close()

    def get_data(self):
        """ Retrieves a data chunk from the socket. """
        try:
            data = self.socket.recv(config.buffer)
        except socket.timeout:
            # It may have finished sending without an error if
            # len(message) % buffer == 0.
            data = None
        except socket.error:
            self.socket_error = True
            data = None
        return data
        
    def parse_request(self, data):
        """ Attempts to load the request, validates it, and calls it. """
        obj = None
        try:
            obj = json.loads(data)
        except ValueError as e:
            return json.dumps(ProtocolError(-32700).generate_error())

        if not obj:
            return json.dumps(ProtocolError(-32600).generate_error())
        batch = True
        if type(obj) is not list:
            batch = False
            obj = [obj,]
        responses = []
        for req in obj:
            request_error = ProtocolError(-32600)
            if type(req) is not dict:
                responses.append(request_error.generate_error())
            elif 'method' not in req.keys():
                responses.append(request_error.generate_error())
            else:
                result = self.parse_call(req)
                if "id" in req:
                    response = generate_response(result, id=req.get('id'))
                    responses.append(response)
        if not responses:
            # It's either a batch of notifications or a single
            # notification, so return nothing.
            return ''
        else:
            if not batch:
                # Single request
                responses = responses[0]
            return json.dumps(responses)
        
    def parse_call(self, obj):
        """
        Parses a JSON request.
        """

        # Get ID, Notification if None
        # This is actually incorrect, as IDs can be null by spec (rare)
        request_id = obj.get('id', None)
        
        # Check for required parameters
        jsonrpc = obj.get('jsonrpc', None)
        method = obj.get('method', None)
        #if not jsonrpc or not method:
        #    return ProtocolError(-32600)
        
        # Validate parameters
        params = obj.get('params', [])
        if type(params) not in (list, dict):
            return ProtocolError(-32602)
        
        # Parse Request
        kwargs = {}
        if type(params) is dict:
            kwargs = params
            params = []
        handler = self.json_request.get_handler(method)
        error_code = None
        message = None
        if handler:
            try:
                response = handler(*params, **kwargs)
                return response
            except Exception:
                logger.error('Error calling handler %s' % method)
                message = traceback.format_exc().splitlines()[-1]
                error_code = -32603
        else:
            error_code = -32601
        return ProtocolError(error_code, message=message)
            
def generate_response(result, **kwargs):
    """
    TODO: Fix so that a request_id can be Null and not a Notification.
    """
    if type(result) is ProtocolError:
        return result.generate_error(**kwargs)
    else:
        response = {'jsonrpc':"2.0", "result":result}
        response.update(kwargs)
        return response
        
def start_server(host, port, handler):
    """
    Wrapper around Server that pre-threads it.
    """
    server = Server((host, port))
    server.add_handler(handler)
    server_thread = threading.Thread(target=server.serve)
    server_thread.daemon = True
    server_thread.start()
    return server
    
def test_server():
    """
    Creates a simple server to be tested against the test_client in
    the client module.
    """
    
    host, port = '', 8080
    
    def echo(message):
        """
        Test method, an example of how simple it *should* be.
        """
        return message
        
    def summation(*args):
        return sum(args)
        
    if '-v' in sys.argv:
        import logging
        config.verbose = True
        logger.addHandler(logging.StreamHandler())
        logger.setLevel(logging.DEBUG)
        
    server = Server((host, port))
    server.add_handler(echo)
    server.add_handler(echo, 'tree.echo')
    server.add_handler(summation, 'sum')
    server_thread = threading.Thread(target=server.serve)
    server_thread.daemon = True
    server_thread.start()
    
    print ("Server running: %s:%s" % (host, port))
    
    try:
        while True:
            time.sleep(0.5)
    except KeyboardInterrupt:
        print('Finished.')
        sys.exit()
    
    
if __name__ == "__main__":
    test_server()



"""
JSONRPCTCP Library default imports
"""

__all__ = ['config', 'history', 'connect', 'logger', 'start_server']

# Set up the basic logging system for JSONRPCTCP.
import logging

class NullLogHandler(logging.Handler):
    """ Ensures that other libraries don't see 'No handler...' output """
    def emit(self, record):
        pass
        
logger = logging.getLogger('JSONRPCTCP')
logger.addHandler(NullLogHandler())

# Default imports
from jsonrpctcp.config import Config
config = Config.instance()
from jsonrpctcp.history import History
history = History.instance()
from jsonrpctcp.client import connect
from jsonrpctcp.server import start_server


"""
This is just the simple config class and singleton for the
JSONRPCTCP library.
"""
import sys
if sys.version_info[0] == 2:
    from ConfigParser import ConfigParser 
else:
    from configparser import ConfigParser 
CONFIG_SECTION = 'jsonrpctcp'
# For encrypting / decrypting the data on keyed connections.
try:
    from Crypto.Cipher import AES
except:
    AES = None

class Config(object):
    """ Simple object to hold jsonrpctcp configuration options """
    _instance = None
    
    def __init__(self):
        """ The default values for the configuration. """
        self.timeout = 5 # seconds
        self.verbose = False
        self.buffer = 4096
        # 'secret' is used by server to indicate encryption --
        # if it is set, encryption is enabled.
        self.secret = None
        # 'crypt' can be any class anything that implements 'new' and 
        # 'encrypt' / 'decrypt' like they Crypto ciphers.
        self.crypt = AES
        # 'crypt_chunk_size' is the size of the message chunk required 
        # by the cipher.
        self.crypt_chunk_size = 16
        # Maximum number of queued connections
        self.max_queue = 10
    
    @classmethod
    def instance(cls):
        """ Retrieves singleton """
        if not cls._instance:
            cls._instance = cls()
        return cls._instance
        
    def load(self, path):
        """ Loads settings from a configuration file. """
        conf = ConfigParser()
        conf.read(path)
        if not conf.has_section(CONFIG_SECTION):
            return
        for option in conf.options(CONFIG_SECTION):
            value = conf.get(CONFIG_SECTION, option)
            orig_type = type(getattr(self, option, None))
            if type(value) is not orig_type and \
                orig_type is not type(None):
                # Doesn't work for False.
                value = orig_type(value)
            setattr(self, option, value)



"""
This is just a list of the Exception classes used, and the
error codes used by them.
"""
from jsonrpctcp import config
import random
import string
import sys
JSONRPC_ERRORS = {
    -32700: {'code':-32700, 'message':'Parse error.'},
    -32600: {'code':-32600, 'message':'Invalid request.'},
    -32601: {'code':-32601, 'message':'Method not found.'},
    -32602: {'code':-32602, 'message':'Invalid parameters.'},
    -32603: {'code':-32603, 'message':'Internal error.'},
}

# The random characters are used for padding the server error messages 
# so that it will hopefully be  harder to brute-force a secret key.
if sys.version_info[0] == 2:
    RANDOM_CHARACTERS = string.letters + string.digits
else:
    RANDOM_CHARACTERS = string.ascii_letters + string.digits
RANDOM_STRING_LENGTH = 12

class ProtocolError(Exception):
    """ Used for system errors and custom errors. """
    
    def __init__(self, code, message=None, data=None):
        message = message or \
            JSONRPC_ERRORS.get(code, {}).get('message', 'Unknown error.')
        self.message = message
        self.code = code
        self.data = data
        
    def generate_error(self, *args, **kwargs):
        """ 
        Return a proper JSON-RPC structure for error messages.
        This also pads a random string on the message to help
        counter brute-forcing "known" messages.
        """
        message = self.message
        if config.secret:
            random_string = ''.join([
                random.choice(RANDOM_CHARACTERS)
                for i in range(RANDOM_STRING_LENGTH)
            ])
            message = '%s (random: %s)' % (self.message, random_string)        
        response = {
            'jsonrpc':"2.0", 
            'error': {
                'message': message,
                'code': self.code
            },
            'id':kwargs.get('id', None)
        }
        return response
        
    def __repr__(self):
        return (
            '<ProtocolError> code:%s, message:%s, data:%s' %
            (self.code, self.message, self.data)
        )

class EncryptionMissing(Exception):
    """ Simple exception if a crypt library is missing """
    pass


from jsonrpctcp import logger

class History(object):
    """
    This holds the response and request objects for a session.
    """
    _instance = None
    
    def __init__(self):
        self.request = None
        self.response = None
    
    @classmethod
    def instance(cls):
        if not cls._instance:
            cls._instance = cls()
        return cls._instance


"""
The Handler class, which can be nested. This should be attached to a
Server instance.
"""

class Handler(object):
    """
    A simple object that should be populated with methods that handle
    the arguments. Should need little interaction with the JSONRequest
    or Server instance.
    """
    
    # def __init__(self, request):
    def __init__(self):
        # self.request = request
        self.__handlers = None
        
    client_address = None

    @property
    def _handlers(self):
        """
        Parses the Handler instance to find all callables which aren't
        protected (starts with '_'), or instances of Handler.
        """
        if not self.__handlers:
            handlers = {}
            for key in dir(self):
                # Underscores are protected
                if key.startswith('_'):
                    continue
                attr = getattr(self, key)
                # Tree syntax
                if issubclass(type(attr), Handler) and attr != self:
                    for name, handler in attr._handlers.items():
                        name = '%s.%s' % (key, name)
                        handlers[name] = handler
                # Normal syntax
                elif hasattr(attr, '__call__'):
                    handlers[key] = attr
            self.__handlers = handlers
        return self.__handlers



"""
These are mostly tests taken verbatim from the JSON RPC v2 Spec document:
http://groups.google.com/group/json-rpc/web/json-rpc-2-0

JSON-RPC v1 tests will be coming soon, as will tests for library "features"
like class translation, etc.

"""
from jsonrpctcp import connect, config, history
from jsonrpctcp.server import Server
from jsonrpctcp import logger
from jsonrpctcp.errors import ProtocolError, EncryptionMissing
import unittest
import os
import time
try:
    import json
except ImportError:
    import simplejson as json
from threading import Thread
import signal
import logging

CLIENT = connect('127.0.0.1', 8000)

class TestCompatibility(unittest.TestCase):
    
    def setUp(self):
        pass 
        
    # Version 2.0 Tests
    
    def test_positional(self):
        """ Positional arguments in a single call """
        result = CLIENT.subtract(23, 42)
        self.assertTrue(result == -19)
        result = CLIENT.subtract(42, 23)
        self.assertTrue(result == 19)
        request = json.loads(history.request)
        response = json.loads(history.response)
        verify_request = {
            "jsonrpc": "2.0", "method": "subtract", 
            "params": [42, 23], "id": request['id']
        }
        verify_response = {
            "jsonrpc": "2.0", "result": 19, "id": request['id']
        }
        self.assertTrue(request == verify_request)
        self.assertTrue(response == verify_response)
        
    def test_named(self):
        """ Named arguments in a single call """
        result = CLIENT.subtract(subtrahend=23, minuend=42)
        self.assertTrue(result == 19)
        result = CLIENT.subtract(minuend=42, subtrahend=23)
        self.assertTrue(result == 19)
        request = json.loads(history.request)
        response = json.loads(history.response)
        verify_request = {
            "jsonrpc": "2.0", "method": "subtract", 
            "params": {"subtrahend": 23, "minuend": 42}, 
            "id": request['id']
        }
        verify_response = {
            "jsonrpc": "2.0", "result": 19, "id": request['id']
        }
        self.assertTrue(request == verify_request)
        self.assertTrue(response == verify_response)
        
    def test_notification(self):
        """ Testing a notification (response should be null) """
        result = CLIENT._notification.update(1, 2, 3, 4, 5)
        self.assertTrue(result == None)
        request = json.loads(history.request)
        response = history.response
        verify_request = {
            "jsonrpc": "2.0", "method": "update", "params": [1,2,3,4,5]
        }
        verify_response = ''
        self.assertTrue(request == verify_request)
        self.assertTrue(response == verify_response)
        
    def test_non_existent_method(self):
        """ Testing a non existent method (raises -32601) """
        self.assertRaises(ProtocolError, CLIENT.foobar)
        request = json.loads(history.request)
        response = json.loads(history.response)
        verify_request = {
            "jsonrpc": "2.0", "method": "foobar", "id": request['id']
        }
        verify_response = {
            "jsonrpc": "2.0", 
            "error": 
                {"code": -32601, "message": response['error']['message']}, 
            "id": request['id']
        }
        self.assertTrue(request == verify_request)
        self.assertTrue(response == verify_response)
        
    def test_invalid_json(self):
        """ Tests an invalid JSON string (raises -32700) """
        invalid_json = '{"jsonrpc": "2.0", "method": "foobar, '+ \
            '"params": "bar", "baz]'
        response = CLIENT._send_and_receive(invalid_json)
        response = json.loads(history.response)
        verify_response = json.loads(
            '{"jsonrpc": "2.0", "error": {"code": -32700,'+
            ' "message": "Parse error."}, "id": null}'
        )
        verify_response['error']['message'] = response['error']['message']
        self.assertTrue(response == verify_response)
        
    def test_invalid_request(self):
        invalid_request = '{"jsonrpc": "2.0", "method": 1, "params": "bar"}'
        response = CLIENT._send_and_receive(invalid_request)
        response = json.loads(history.response)
        verify_response = json.loads(
            '{"jsonrpc": "2.0", "error": {"code": -32600, '+
            '"message": "Invalid Request."}, "id": null}'
        )
        verify_response['error']['message'] = response['error']['message']
        self.assertTrue(response == verify_response)
        
    def test_batch_invalid_json(self):
        invalid_request = '[ {"jsonrpc": "2.0", "method": "sum", '+ \
            '"params": [1,2,4], "id": "1"},{"jsonrpc": "2.0", "method" ]'
        response = CLIENT._send_and_receive(
            invalid_request, batch=True
        )
        response = json.loads(history.response)
        verify_response = json.loads(
            '{"jsonrpc": "2.0", "error": {"code": -32700,'+
            '"message": "Parse error."}, "id": null}'
        )
        verify_response['error']['message'] = response['error']['message']
        self.assertTrue(response == verify_response)
        
    def test_empty_array(self):
        invalid_request = '[]'
        response = CLIENT._send_and_receive(invalid_request)
        response = json.loads(history.response)
        verify_response = json.loads(
            '{"jsonrpc": "2.0", "error": {"code": -32600, '+
            '"message": "Invalid Request."}, "id": null}'
        )
        verify_response['error']['message'] = response['error']['message']
        self.assertTrue(response == verify_response)
        
    def test_nonempty_array(self):
        invalid_request = '[1,2]'
        request_obj = json.loads(invalid_request)
        response = CLIENT._send_and_receive(invalid_request)
        response = json.loads(history.response)
        self.assertTrue(len(response) == len(request_obj))
        for resp in response:
            verify_resp = json.loads(
                '{"jsonrpc": "2.0", "error": {"code": -32600, '+
                '"message": "Invalid Request."}, "id": null}'
            )
            verify_resp['error']['message'] = resp['error']['message']
            self.assertTrue(resp == verify_resp)
        
    def test_batch(self):
        multicall = CLIENT._batch()
        multicall.sum(1,2,4)
        multicall._notification.notify_hello(7)
        multicall.subtract(42,23)
        multicall.foo.get(name='myself')
        multicall.get_data()
        job_requests = [j._request() for j in multicall._requests]
        job_requests.insert(3, {"foo": "boo"})
        json_requests = '[%s]' % ','.join(
            map(lambda x:json.dumps(x), job_requests)
        )
        requests = json.loads(json_requests)
        response_text = CLIENT._send_and_receive(json_requests, batch=True)
        responses = json.loads(response_text)
        
        verify_requests = json.loads("""[
            {"jsonrpc": "2.0", "method": "sum", "params": [1,2,4], "id": "1"},
            {"jsonrpc": "2.0", "method": "notify_hello", "params": [7]},
            {"jsonrpc": "2.0", "method": "subtract", "params": [42,23], "id": "2"},
            {"foo": "boo"},
            {"jsonrpc": "2.0", "method": "foo.get", "params": {"name": "myself"}, "id": "5"},
            {"jsonrpc": "2.0", "method": "get_data", "id": "9"} 
        ]""")
            
        # Thankfully, these are in order so testing is pretty simple.
        verify_responses = json.loads("""[
            {"jsonrpc": "2.0", "result": 7, "id": "1"},
            {"jsonrpc": "2.0", "result": 19, "id": "2"},
            {"jsonrpc": "2.0", "error": {"code": -32600, "message": "Invalid Request."}, "id": null},
            {"jsonrpc": "2.0", "error": {"code": -32601, "message": "Method not found."}, "id": "5"},
            {"jsonrpc": "2.0", "result": ["hello", 5], "id": "9"}
        ]""")
        
        self.assertTrue(len(requests) == len(verify_requests))
        self.assertTrue(len(responses) == len(verify_responses))
        
        responses_by_id = {}
        response_i = 0
        
        for i in range(len(requests)):
            verify_request = verify_requests[i]
            request = requests[i]
            response = None
            if request.get('method') != 'notify_hello':
                req_id = request.get('id')
                if verify_request.has_key('id'):
                    verify_request['id'] = req_id
                verify_response = verify_responses[response_i]
                verify_response['id'] = req_id
                responses_by_id[req_id] = verify_response
                response_i += 1
                response = verify_response
            self.assertTrue(request == verify_request)
            
        for response in responses:
            verify_response = responses_by_id.get(response.get('id'))
            if verify_response.has_key('error'):
                verify_response['error']['message'] = \
                    response['error']['message']
            self.assertTrue(response == verify_response)
        
    def test_batch_notifications(self): 
        multicall = CLIENT._batch()
        multicall._notification.notify_sum(1, 2, 4)
        multicall._notification.notify_hello(7)
        results = multicall()
        result_list = []
        for result in results:
            result_list.append(result)
        self.assertTrue(len(result_list) == 0)
        valid_request = json.loads(
            '[{"jsonrpc": "2.0", "method": "notify_sum", '+
            '"params": [1,2,4]},{"jsonrpc": "2.0", '+
            '"method": "notify_hello", "params": [7]}]'
        )
        request = json.loads(history.request)
        self.assertTrue(len(request) == len(valid_request))
        for i in range(len(request)):
            req = request[i]
            valid_req = valid_request[i]
            self.assertTrue(req == valid_req)
        self.assertTrue(history.response == '')
        
    # Other tests
    
    def test_namespace(self):
        response = CLIENT.namespace.sum(1,2,4)
        request = json.loads(history.request)
        response = json.loads(history.response)
        verify_request = {
            "jsonrpc": "2.0", "params": [1, 2, 4], 
            "id": "5", "method": "namespace.sum"
        }
        verify_response = {
            "jsonrpc": "2.0", "result": 7, "id": "5"
        }
        verify_request['id'] = request['id']
        verify_response['id'] = request['id']
        self.assertTrue(verify_request == request)
        self.assertTrue(verify_response == response)
        
class TestEncryption(unittest.TestCase):
    
    def setUp(self):
        config.secret = '12345abcdef67890'
    
    def test_no_encryption(self):
        crypt = config.crypt
        config.crypt = None
        self.assertRaises(
            EncryptionMissing, connect, 'localhost', 8001, config.secret
        )
        config.crypt = crypt
        
    def test_encryption(self):
        client = connect('localhost', 8001, config.secret)
        result = client.sum(49, 51)
        self.assertTrue(result == 100)
        
    def tearDown(self):
        config.secret = None
        
""" Test Methods """
def subtract(minuend, subtrahend):
    """ Using the keywords from the JSON-RPC v2 doc """
    return minuend-subtrahend
    
def update(*args):
    return args
    
def summation(*args):
    return sum(args)
    
def notify_hello(*args):
    return args
    
def get_data():
    return ['hello', 5]
        
def test_set_up():
    # Because 'setUp' on unittests are called multiple times
    # and starting a server each time is inefficient / a headache
    
    # Starting normal server
    server = Server(('', 8000))
    server.add_handler(summation, 'sum')
    server.add_handler(summation, 'notify_sum')
    server.add_handler(notify_hello)
    server.add_handler(subtract)
    server.add_handler(update)
    server.add_handler(get_data)
    server.add_handler(summation, 'namespace.sum')
    server_proc = Thread(target=server.serve)
    server_proc.daemon = True
    server_proc.start()
    
    #Starting secure server
    server2 = Server(('', 8001))
    server2.add_handler(summation, 'sum')
    server_proc2 = Thread(target=server2.serve)
    server_proc2.daemon = True
    server_proc2.start()
    
    time.sleep(1) # give it time to start up
    #logger.setLevel(logging.DEBUG)
    #logger.addHandler(logging.StreamHandler())

if __name__ == '__main__':
    test_set_up()
    unittest.main()
    



#!/usr/bin/env python

from distutils.core import setup

setup(
    name='jsonrpctcp',
    version='0.3',
    description='JSON-RPC (v2) over TCP sockets.',
    author='Josh Marshall',
    author_email='catchjosh@gmail.com',
    packages=['jsonrpctcp',]
)

